@@ -56,6 +56,8 @@ int main_polysomy(int argc, char *argv[]);
 #endif
 int main_plugin(int argc, char *argv[]);
 int main_consensus(int argc, char *argv[]);
+int main_csq(int argc, char *argv[]);
+int bam_mpileup(int argc, char *argv[]);
 
 typedef struct
 {
@@ -142,6 +144,10 @@ static cmd_t cmds[] =
       .alias = "cnv",
       .help  = "HMM CNV calling"
     },
+    { .func  = main_csq,
+      .alias = "csq",
+      .help  = "call variation consequences"
+    },
     { .func  = main_vcffilter,
       .alias = "filter",
       .help  = "filter VCF/BCF files using fixed thresholds"
@@ -150,6 +156,10 @@ static cmd_t cmds[] =
       .alias = "gtcheck",
       .help  = "check sample concordance, detect sample swaps and contamination"
     },
+    { .func  = bam_mpileup,
+        .alias = "mpileup",
+        .help  = "multi-way pileup producing genotype likelihoods"
+    },
 #if USE_GPL
     { .func  = main_polysomy,
       .alias = "polysomy",
