@@ -491,7 +491,7 @@ int main_pad2unpad(int argc, char *argv[])
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
 
     static const struct option lopts[] = {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 'T'),
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 'T', '-'),
         { NULL, 0, NULL, 0 }
     };
 
@@ -603,7 +603,7 @@ static int usage(int is_long_help)
     fprintf(stderr, "               Padded reference sequence file [null]\n");
     fprintf(stderr, "  -o FILE      Output file name [stdout]\n");
     fprintf(stderr, "  -?           Longer help\n");
-    sam_global_opt_help(stderr, "-...-");
+    sam_global_opt_help(stderr, "-...--");
 
     if (is_long_help)
         fprintf(stderr,
