@@ -493,7 +493,7 @@ int main_pad2unpad(int argc, char *argv[])
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
 
     static const struct option lopts[] = {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 'T'),
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 'T', '-'),
         { NULL, 0, NULL, 0 }
     };
 
@@ -605,7 +605,7 @@ static int usage(int is_long_help)
     fprintf(pysam_stderr, "               Padded reference sequence file [null]\n");
     fprintf(pysam_stderr, "  -o FILE      Output file name [pysam_stdout]\n");
     fprintf(pysam_stderr, "  -?           Longer help\n");
-    sam_global_opt_help(pysam_stderr, "-...-");
+    sam_global_opt_help(pysam_stderr, "-...--");
 
     if (is_long_help)
         fprintf(pysam_stderr,
