@@ -3,7 +3,7 @@
 /*  phase.c -- phase subcommand.
 
     Copyright (C) 2011 Broad Institute.
-    Copyright (C) 2013, 2014 Genome Research Ltd.
+    Copyright (C) 2013-2016 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk>
 
@@ -25,15 +25,19 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdint.h>
 #include <math.h>
 #include <zlib.h>
+#include "htslib/hts.h"
 #include "htslib/sam.h"
-#include "errmod.h"
+#include "htslib/kstring.h"
 #include "sam_opts.h"
+#include "samtools.h"
 
 #include "htslib/kseq.h"
 KSTREAM_INIT(gzFile, gzread, 16384)
@@ -55,6 +59,7 @@ typedef struct {
     samFile* fp;
     bam_hdr_t* fp_hdr;
     char *pre;
+    char *out_name[3];
     samFile* out[3];
     bam_hdr_t* out_hdr[3];
     // alignment queue
@@ -335,7 +340,7 @@ static int clean_seqs(int vpos, nseq_t *hash)
     return ret;
 }
 
-static void dump_aln(phaseg_t *g, int min_pos, const nseq_t *hash)
+static int dump_aln(phaseg_t *g, int min_pos, const nseq_t *hash)
 {
     int i, is_flip, drop_ambi;
     drop_ambi = g->flag & FLAG_DROP_AMBI;
@@ -363,12 +368,16 @@ static void dump_aln(phaseg_t *g, int min_pos, const nseq_t *hash)
             if (which < 2 && is_flip) which = 1 - which; // increase the randomness
         }
         if (which == 3) which = (drand48() < 0.5);
-        sam_write1(g->out[which], g->out_hdr[which], b);
+        if (sam_write1(g->out[which], g->out_hdr[which], b) < 0) {
+            print_error_errno("phase", "error writing to '%s'", g->out_name[which]);
+            return -1;
+        }
         bam_destroy1(b);
         g->b[i] = 0;
     }
     memmove(g->b, g->b + i, (g->n - i) * sizeof(void*));
     g->n -= i;
+    return 0;
 }
 
 static int phase(phaseg_t *g, const char *chr, int vpos, uint64_t *cns, nseq_t *hash)
@@ -383,8 +392,8 @@ static int phase(phaseg_t *g, const char *chr, int vpos, uint64_t *cns, nseq_t *
     i = clean_seqs(vpos, hash); // i is true if hash has an element with its vpos >= vpos
     min_pos = i? cns[vpos]>>32 : 0x7fffffff;
     if (vpos == 1) {
-        printf("PS\t%s\t%d\t%d\n", chr, (int)(cns[0]>>32) + 1, (int)(cns[0]>>32) + 1);
-        printf("M0\t%s\t%d\t%d\t%c\t%c\t%d\t0\t0\t0\t0\n//\n", chr, (int)(cns[0]>>32) + 1, (int)(cns[0]>>32) + 1,
+        fprintf(pysam_stdout, "PS\t%s\t%d\t%d\n", chr, (int)(cns[0]>>32) + 1, (int)(cns[0]>>32) + 1);
+        fprintf(pysam_stdout, "M0\t%s\t%d\t%d\t%c\t%c\t%d\t0\t0\t0\t0\n//\n", chr, (int)(cns[0]>>32) + 1, (int)(cns[0]>>32) + 1,
             "ACGTX"[cns[0]&3], "ACGTX"[cns[0]>>16&3], g->vpos_shift + 1);
         for (k = 0; k < kh_end(hash); ++k) {
             if (kh_exist(hash, k)) {
@@ -395,14 +404,14 @@ static int phase(phaseg_t *g, const char *chr, int vpos, uint64_t *cns, nseq_t *
                 else f->phased = 1, f->phase = f->seq[0] - 1;
             }
         }
-        dump_aln(g, min_pos, hash);
+        if (dump_aln(g, min_pos, hash) < 0) return -1;
         ++g->vpos_shift;
         return 1;
     }
     { // phase
         int **cnt;
         uint64_t *mask;
-        printf("PS\t%s\t%d\t%d\n", chr, (int)(cns[0]>>32) + 1, (int)(cns[vpos-1]>>32) + 1);
+        fprintf(pysam_stdout, "PS\t%s\t%d\t%d\n", chr, (int)(cns[0]>>32) + 1, (int)(cns[vpos-1]>>32) + 1);
         sitemask = calloc(vpos, 1);
         cnt = count_all(g->k, vpos, hash);
         path = dynaprog(g->k, vpos, cnt);
@@ -423,13 +432,13 @@ static int phase(phaseg_t *g, const char *chr, int vpos, uint64_t *cns, nseq_t *
         }
     }
     for (i = 0; i < n_masked; ++i)
-        printf("FL\t%s\t%d\t%d\n", chr, (int)(regmask[i]>>32) + 1, (int)regmask[i] + 1);
+        fprintf(pysam_stdout, "FL\t%s\t%d\t%d\n", chr, (int)(regmask[i]>>32) + 1, (int)regmask[i] + 1);
     for (i = 0; i < vpos; ++i) {
         uint64_t x = pcnt[i];
         int8_t c[2];
         c[0] = (cns[i]&0xffff)>>2 == 0? 4 : (cns[i]&3);
         c[1] = (cns[i]>>16&0xffff)>>2 == 0? 4 : (cns[i]>>16&3);
-        printf("M%d\t%s\t%d\t%d\t%c\t%c\t%d\t%d\t%d\t%d\t%d\n", sitemask[i]+1, chr, (int)(cns[0]>>32) + 1, (int)(cns[i]>>32) + 1, "ACGTX"[c[path[i]]], "ACGTX"[c[1-path[i]]],
+        fprintf(pysam_stdout, "M%d\t%s\t%d\t%d\t%c\t%c\t%d\t%d\t%d\t%d\t%d\n", sitemask[i]+1, chr, (int)(cns[0]>>32) + 1, (int)(cns[i]>>32) + 1, "ACGTX"[c[path[i]]], "ACGTX"[c[1-path[i]]],
             i + g->vpos_shift + 1, (int)(x&0xffff), (int)(x>>16&0xffff), (int)(x>>32&0xffff), (int)(x>>48&0xffff));
     }
     free(path); free(pcnt); free(regmask); free(sitemask);
@@ -441,19 +450,19 @@ static int phase(phaseg_t *g, const char *chr, int vpos, uint64_t *cns, nseq_t *
     ks_introsort_rseq(n_seqs, seqs);
     for (i = 0; i < n_seqs; ++i) {
         frag_t *f = seqs[i];
-        printf("EV\t0\t%s\t%d\t40\t%dM\t*\t0\t0\t", chr, f->vpos + 1 + g->vpos_shift, f->vlen);
+        fprintf(pysam_stdout, "EV\t0\t%s\t%d\t40\t%dM\t*\t0\t0\t", chr, f->vpos + 1 + g->vpos_shift, f->vlen);
         for (j = 0; j < f->vlen; ++j) {
             uint32_t c = cns[f->vpos + j];
-            if (f->seq[j] == 0) putchar('N');
-            else putchar("ACGT"[f->seq[j] == 1? (c&3) : (c>>16&3)]);
+            if (f->seq[j] == 0) fputc('N', pysam_stdout);
+            else fputc("ACGT"[f->seq[j] == 1? (c&3) : (c>>16&3)], pysam_stdout);
         }
-        printf("\t*\tYP:i:%d\tYF:i:%d\tYI:i:%d\tYO:i:%d\tYS:i:%d\n", f->phase, f->flip, f->in, f->out, f->beg+1);
+        fprintf(pysam_stdout, "\t*\tYP:i:%d\tYF:i:%d\tYI:i:%d\tYO:i:%d\tYS:i:%d\n", f->phase, f->flip, f->in, f->out, f->beg+1);
     }
     free(seqs);
-    printf("//\n");
-    fflush(stdout);
+    fprintf(pysam_stdout, "//\n");
+    fflush(pysam_stdout);
     g->vpos_shift += vpos;
-    dump_aln(g, min_pos, hash);
+    if (dump_aln(g, min_pos, hash) < 0) return -1;
     return vpos;
 }
 
@@ -538,6 +547,26 @@ static int gl2cns(float q[16])
     return (min_ij>>2&3) == (min_ij&3)? 0 : 1<<18 | (min_ij>>2&3)<<16 | (min_ij&3) | (int)(min2 - min + .499) << 2;
 }
 
+static int start_output(phaseg_t *g, int c, const char *middle, const htsFormat *fmt)
+{
+    kstring_t s = { 0, 0, NULL };
+    ksprintf(&s, "%s.%s.%s", g->pre, middle, hts_format_file_extension(fmt));
+    g->out_name[c] = ks_release(&s);
+    g->out[c] = sam_open_format(g->out_name[c], "wb", fmt);
+    if (! g->out[c]) {
+        print_error_errno("phase", "Failed to open output file '%s'", g->out_name[c]);
+        return -1;
+    }
+
+    g->out_hdr[c] = bam_hdr_dup(g->fp_hdr);
+    if (sam_hdr_write(g->out[c], g->out_hdr[c]) < 0) {
+        print_error_errno("phase", "Failed to write header for '%s'", g->out_name[c]);
+        return -1;
+    }
+
+    return 0;
+}
+
 int main_phase(int argc, char *argv[])
 {
     int c, tid, pos, vpos = 0, n, lasttid = -1, max_vpos = 0, usage = 0;
@@ -553,10 +582,12 @@ int main_phase(int argc, char *argv[])
 
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
     static const struct option lopts[] = {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 0),
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 0, '-'),
         { NULL, 0, NULL, 0 }
     };
 
+    // FIXME Leaks galore in the case of error returns
+
     memset(&g, 0, sizeof(phaseg_t));
     g.flag = FLAG_FIX_CHIMERA;
     g.min_varLOD = 37; g.k = 13; g.min_baseQ = 13; g.max_depth = 256;
@@ -578,27 +609,32 @@ int main_phase(int argc, char *argv[])
         if (usage) break;
     }
     if (usage || argc == optind) {
-        fprintf(pysamerr, "\n");
-        fprintf(pysamerr, "Usage:   samtools phase [options] <in.bam>\n\n");
-        fprintf(pysamerr, "Options: -k INT    block length [%d]\n", g.k);
-        fprintf(pysamerr, "         -b STR    prefix of BAMs to output [null]\n");
-        fprintf(pysamerr, "         -q INT    min het phred-LOD [%d]\n", g.min_varLOD);
-        fprintf(pysamerr, "         -Q INT    min base quality in het calling [%d]\n", g.min_baseQ);
-        fprintf(pysamerr, "         -D INT    max read depth [%d]\n", g.max_depth);
-//      fprintf(pysamerr, "         -l FILE   list of sites to phase [null]\n");
-        fprintf(pysamerr, "         -F        do not attempt to fix chimeras\n");
-        fprintf(pysamerr, "         -A        drop reads with ambiguous phase\n");
-//      fprintf(pysamerr, "         -e        do not discover SNPs (effective with -l)\n");
-        fprintf(pysamerr, "\n");
+        fprintf(pysam_stderr, "\n");
+        fprintf(pysam_stderr, "Usage:   samtools phase [options] <in.bam>\n\n");
+        fprintf(pysam_stderr, "Options: -k INT    block length [%d]\n", g.k);
+        fprintf(pysam_stderr, "         -b STR    prefix of BAMs to output [null]\n");
+        fprintf(pysam_stderr, "         -q INT    min het phred-LOD [%d]\n", g.min_varLOD);
+        fprintf(pysam_stderr, "         -Q INT    min base quality in het calling [%d]\n", g.min_baseQ);
+        fprintf(pysam_stderr, "         -D INT    max read depth [%d]\n", g.max_depth);
+//      fprintf(pysam_stderr, "         -l FILE   list of sites to phase [null]\n");
+        fprintf(pysam_stderr, "         -F        do not attempt to fix chimeras\n");
+        fprintf(pysam_stderr, "         -A        drop reads with ambiguous phase\n");
+//      fprintf(pysam_stderr, "         -e        do not discover SNPs (effective with -l)\n");
+        fprintf(pysam_stderr, "\n");
 
-        sam_global_opt_help(pysamerr, "-....");
+        sam_global_opt_help(pysam_stderr, "-....-");
 
         return 1;
     }
     g.fp = sam_open_format(argv[optind], "r", &ga.in);
+    if (!g.fp) {
+        print_error_errno("phase", "Couldn't open '%s'", argv[optind]);
+        return 1;
+    }
     g.fp_hdr = sam_hdr_read(g.fp);
     if (g.fp_hdr == NULL) {
-        fprintf(pysamerr, "Failed to read header for '%s'\n", argv[optind]);
+        fprintf(pysam_stderr, "[%s] Failed to read header for '%s'\n",
+                __func__, argv[optind]);
         return 1;
     }
     if (fn_list) { // read the list of sites to phase
@@ -606,20 +642,13 @@ int main_phase(int argc, char *argv[])
         free(fn_list);
     } else g.flag &= ~FLAG_LIST_EXCL;
     if (g.pre) { // open BAMs to write
-        char *s = (char*)malloc(strlen(g.pre) + 20);
         if (ga.out.format == unknown_format)
             ga.out.format = bam; // default via "wb".
-        strcpy(s, g.pre); strcat(s, ".0."); strcat(s, hts_format_file_extension(&ga.out));
-        g.out[0] = sam_open_format(s, "wb", &ga.out);
-        strcpy(s, g.pre); strcat(s, ".1."); strcat(s, hts_format_file_extension(&ga.out));
-        g.out[1] = sam_open_format(s, "wb", &ga.out);
-        strcpy(s, g.pre); strcat(s, ".chimera."); strcat(s, hts_format_file_extension(&ga.out));
-        g.out[2] = sam_open_format(s, "wb", &ga.out);
-        for (c = 0; c <= 2; ++c) {
-            g.out_hdr[c] = bam_hdr_dup(g.fp_hdr);
-            sam_hdr_write(g.out[c], g.out_hdr[c]);
-        }
-        free(s);
+
+        // Open each output file g.out[0..2], dupping and writing the header
+        if (start_output(&g, 0, "0", &ga.out) < 0 ||
+            start_output(&g, 1, "1", &ga.out) < 0 ||
+            start_output(&g, 2, "chimera", &ga.out) < 0) return 1;
     }
 
     iter = bam_plp_init(readaln, &g);
@@ -627,20 +656,20 @@ int main_phase(int argc, char *argv[])
     seqs = kh_init(64);
     em = errmod_init(1. - 0.83);
     bases = calloc(g.max_depth, 2);
-    printf("CC\n");
-    printf("CC\tDescriptions:\nCC\n");
-    printf("CC\t  CC      comments\n");
-    printf("CC\t  PS      start of a phase set\n");
-    printf("CC\t  FL      filtered region\n");
-    printf("CC\t  M[012]  markers; 0 for singletons, 1 for phased and 2 for filtered\n");
-    printf("CC\t  EV      supporting reads; SAM format\n");
-    printf("CC\t  //      end of a phase set\nCC\n");
-    printf("CC\tFormats of PS, FL and M[012] lines (1-based coordinates):\nCC\n");
-    printf("CC\t  PS  chr  phaseSetStart  phaseSetEnd\n");
-    printf("CC\t  FL  chr  filterStart    filterEnd\n");
-    printf("CC\t  M?  chr  PS  pos  allele0  allele1  hetIndex  #supports0  #errors0  #supp1  #err1\n");
-    printf("CC\nCC\n");
-    fflush(stdout);
+    fprintf(pysam_stdout, "CC\n");
+    fprintf(pysam_stdout, "CC\tDescriptions:\nCC\n");
+    fprintf(pysam_stdout, "CC\t  CC      comments\n");
+    fprintf(pysam_stdout, "CC\t  PS      start of a phase set\n");
+    fprintf(pysam_stdout, "CC\t  FL      filtered region\n");
+    fprintf(pysam_stdout, "CC\t  M[012]  markers; 0 for singletons, 1 for phased and 2 for filtered\n");
+    fprintf(pysam_stdout, "CC\t  EV      supporting reads; SAM format\n");
+    fprintf(pysam_stdout, "CC\t  //      end of a phase set\nCC\n");
+    fprintf(pysam_stdout, "CC\tFormats of PS, FL and M[012] lines (1-based coordinates):\nCC\n");
+    fprintf(pysam_stdout, "CC\t  PS  chr  phaseSetStart  phaseSetEnd\n");
+    fprintf(pysam_stdout, "CC\t  FL  chr  filterStart    filterEnd\n");
+    fprintf(pysam_stdout, "CC\t  M?  chr  PS  pos  allele0  allele1  hetIndex  #supports0  #errors0  #supp1  #err1\n");
+    fprintf(pysam_stdout, "CC\nCC\n");
+    fflush(pysam_stdout);
     while ((plp = bam_plp_auto(iter, &tid, &pos, &n)) != 0) {
         int i, k, c, tmp, dophase = 1, in_set = 0;
         float q[16];
@@ -649,7 +678,10 @@ int main_phase(int argc, char *argv[])
             g.vpos_shift = 0;
             if (lasttid >= 0) {
                 seqs = shrink_hash(seqs);
-                phase(&g, g.fp_hdr->target_name[lasttid], vpos, cns, seqs);
+                if (phase(&g, g.fp_hdr->target_name[lasttid],
+                          vpos, cns, seqs) < 0) {
+                    return 1;
+                }
                 update_vpos(0x7fffffff, seqs);
             }
             lasttid = tid;
@@ -718,14 +750,20 @@ int main_phase(int argc, char *argv[])
         }
         if (dophase) {
             seqs = shrink_hash(seqs);
-            phase(&g, g.fp_hdr->target_name[tid], vpos, cns, seqs);
+            if (phase(&g, g.fp_hdr->target_name[tid], vpos, cns, seqs) < 0) {
+                return 1;
+            }
             update_vpos(vpos, seqs);
             cns[0] = cns[vpos];
             vpos = 0;
         }
         ++vpos;
     }
-    if (tid >= 0) phase(&g, g.fp_hdr->target_name[tid], vpos, cns, seqs);
+    if (tid >= 0) {
+        if (phase(&g, g.fp_hdr->target_name[tid], vpos, cns, seqs) < 0) {
+            return 1;
+        }
+    }
     bam_hdr_destroy(g.fp_hdr);
     bam_plp_destroy(iter);
     sam_close(g.fp);
@@ -735,11 +773,18 @@ int main_phase(int argc, char *argv[])
     errmod_destroy(em);
     free(bases);
     if (g.pre) {
+        int res = 0;
         for (c = 0; c <= 2; ++c) {
-            sam_close(g.out[c]);
+            if (sam_close(g.out[c]) < 0) {
+                fprintf(pysam_stderr, "[%s] error on closing '%s'\n",
+                        __func__, g.out_name[c]);
+                res = 1;
+            }
             bam_hdr_destroy(g.out_hdr[c]);
+            free(g.out_name[c]);
         }
         free(g.pre); free(g.b);
+        if (res) return 1;
     }
     sam_global_args_free(&ga);
     return 0;
