@@ -24,6 +24,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include "../../bam_split.c"
 #include "../test.h"
 #include <unistd.h>
@@ -42,7 +44,8 @@ void setup_test_1(bam_hdr_t** hdr_in)
 bool check_test_1(const bam_hdr_t* hdr) {
     const char *test1_res =
     "@HD\tVN:1.4\n"
-    "@SQ\tSN:blah\n";
+    "@SQ\tSN:blah\n"
+    "@PG\tID:samtools\tPN:samtools\tVN:x.y.test\tCL:test_filter_header_rg foo bar baz\n";
 
     if (strcmp(hdr->text, test1_res)) {
         return false;
@@ -65,7 +68,8 @@ bool check_test_2(const bam_hdr_t* hdr) {
     const char *test2_res =
     "@HD\tVN:1.4\n"
     "@SQ\tSN:blah\n"
-    "@RG\tID:fish\n";
+    "@RG\tID:fish\n"
+    "@PG\tID:samtools\tPN:samtools\tVN:x.y.test\tCL:test_filter_header_rg foo bar baz\n";
 
     if (strcmp(hdr->text, test2_res)) {
         return false;
@@ -73,7 +77,7 @@ bool check_test_2(const bam_hdr_t* hdr) {
     return true;
 }
 
-int main(int argc, char**argv)
+int samtools_test_filter_header_rg_main(int argc, char *argv[])
 {
     // test state
     const int NUM_TESTS = 2;
@@ -82,13 +86,15 @@ int main(int argc, char**argv)
     int failure = 0;
 
     int getopt_char;
+    char *test_argv[] = { "test_filter_header_rg", "foo\tbar", "baz" };
+    char *arg_list = stringify_argv(3, test_argv);
     while ((getopt_char = getopt(argc, argv, "v")) != -1) {
         switch (getopt_char) {
             case 'v':
                 ++verbose;
                 break;
             default:
-                printf(
+                fprintf(pysam_stdout, 
                        "usage: test_filter_header_rg [-v]\n\n"
                        " -v verbose output\n"
                        );
@@ -97,31 +103,31 @@ int main(int argc, char**argv)
     }
 
 
-    // Setup pysamerr redirect
+    // Setup pysam_stderr redirect
     kstring_t res = { 0, 0, NULL };
-    FILE* orig_pysamerr = fdopen(dup(STDERR_FILENO), "a"); // Save pysamerr
+    FILE* orig_pysam_stderr = fdopen(dup(STDERR_FILENO), "a"); // Save pysam_stderr
     char* tempfname = (optind < argc)? argv[optind] : "test_count_rg.tmp";
     FILE* check = NULL;
 
     // setup
-    if (verbose) printf("BEGIN test 1\n");  // test eliminating a tag that isn't there
+    if (verbose) fprintf(pysam_stdout, "BEGIN test 1\n");  // test eliminating a tag that isn't there
     bam_hdr_t* hdr1;
     const char* id_to_keep_1 = "1#2.3";
     setup_test_1(&hdr1);
     if (verbose > 1) {
-        printf("hdr1\n");
+        fprintf(pysam_stdout, "hdr1\n");
         dump_hdr(hdr1);
     }
-    if (verbose) printf("RUN test 1\n");
+    if (verbose) fprintf(pysam_stdout, "RUN test 1\n");
 
     // test
-    xfreopen(tempfname, "w", pysamerr); // Redirect pysamerr to pipe
-    bool result_1 = filter_header_rg(hdr1, id_to_keep_1);
-    fclose(pysamerr);
+    xfreopen(tempfname, "w", pysam_stderr); // Redirect pysam_stderr to pipe
+    bool result_1 = filter_header_rg(hdr1, id_to_keep_1, arg_list);
+    fclose(pysam_stderr);
 
-    if (verbose) printf("END RUN test 1\n");
+    if (verbose) fprintf(pysam_stdout, "END RUN test 1\n");
     if (verbose > 1) {
-        printf("hdr1\n");
+        fprintf(pysam_stdout, "hdr1\n");
         dump_hdr(hdr1);
     }
 
@@ -135,32 +141,32 @@ int main(int argc, char**argv)
         ++success;
     } else {
         ++failure;
-        if (verbose) printf("FAIL test 1\n");
+        if (verbose) fprintf(pysam_stdout, "FAIL test 1\n");
     }
     fclose(check);
 
     // teardown
     bam_hdr_destroy(hdr1);
-    if (verbose) printf("END test 1\n");
+    if (verbose) fprintf(pysam_stdout, "END test 1\n");
 
-    if (verbose) printf("BEGIN test 2\n");  // test eliminating a tag that is there
+    if (verbose) fprintf(pysam_stdout, "BEGIN test 2\n");  // test eliminating a tag that is there
     bam_hdr_t* hdr2;
     const char* id_to_keep_2 = "fish";
     setup_test_2(&hdr2);
     if (verbose > 1) {
-        printf("hdr2\n");
+        fprintf(pysam_stdout, "hdr2\n");
         dump_hdr(hdr2);
     }
-    if (verbose) printf("RUN test 2\n");
+    if (verbose) fprintf(pysam_stdout, "RUN test 2\n");
 
     // test
-    xfreopen(tempfname, "w", pysamerr); // Redirect pysamerr to pipe
-    bool result_2 = filter_header_rg(hdr2, id_to_keep_2);
-    fclose(pysamerr);
+    xfreopen(tempfname, "w", pysam_stderr); // Redirect pysam_stderr to pipe
+    bool result_2 = filter_header_rg(hdr2, id_to_keep_2, arg_list);
+    fclose(pysam_stderr);
 
-    if (verbose) printf("END RUN test 2\n");
+    if (verbose) fprintf(pysam_stdout, "END RUN test 2\n");
     if (verbose > 1) {
-        printf("hdr2\n");
+        fprintf(pysam_stdout, "hdr2\n");
         dump_hdr(hdr2);
     }
 
@@ -174,21 +180,22 @@ int main(int argc, char**argv)
         ++success;
     } else {
         ++failure;
-        if (verbose) printf("FAIL test 2\n");
+        if (verbose) fprintf(pysam_stdout, "FAIL test 2\n");
     }
     fclose(check);
 
     // teardown
     bam_hdr_destroy(hdr2);
-    if (verbose) printf("END test 2\n");
+    if (verbose) fprintf(pysam_stdout, "END test 2\n");
 
 
     // Cleanup
     free(res.s);
+    free(arg_list);
     remove(tempfname);
     if (failure > 0)
-        fprintf(orig_pysamerr, "%d failures %d successes\n", failure, success);
-    fclose(orig_pysamerr);
+        fprintf(orig_pysam_stderr, "%d failures %d successes\n", failure, success);
+    fclose(orig_pysam_stderr);
 
     return (success == NUM_TESTS)? EXIT_SUCCESS : EXIT_FAILURE;
 }
