@@ -46,19 +46,19 @@ class TestAlignedSegment(ReadTest):
         self.assertEqual(a.query_sequence, None)
         self.assertEqual(pysam.qualities_to_qualitystring(a.query_qualities), None)
         self.assertEqual(a.flag, 0)
-        self.assertEqual(a.reference_id, 0)
+        self.assertEqual(a.reference_id, -1)
         self.assertEqual(a.mapping_quality, 0)
         self.assertEqual(a.cigartuples, None)
         self.assertEqual(a.tags, [])
-        self.assertEqual(a.next_reference_id, 0)
-        self.assertEqual(a.next_reference_start, 0)
+        self.assertEqual(a.next_reference_id, -1)
+        self.assertEqual(a.next_reference_start, -1)
         self.assertEqual(a.template_length, 0)
 
     def testStrOfEmptyRead(self):
         a = pysam.AlignedSegment()
         s = str(a)
         self.assertEqual(
-            "None\t0\t0\t0\t0\tNone\t0\t0\t0\tNone\tNone\t[]",
+            "None\t0\t-1\t-1\t0\tNone\t-1\t-1\t0\tNone\tNone\t[]",
             s)
 
     def testSettingTagInEmptyRead(self):
@@ -231,6 +231,50 @@ class TestAlignedSegment(ReadTest):
         self.assertEqual(a.get_blocks(),
                          [(20, 30), (31, 40), (40, 60)])
 
+    def test_infer_query_length(self):
+        '''Test infer_query_length on M|=|X|I|D|H|S cigar ops'''
+        a = self.buildRead()
+        a.cigarstring = '40M'
+        self.assertEqual(a.infer_query_length(), 40)
+        a.cigarstring = '40='
+        self.assertEqual(a.infer_query_length(), 40)
+        a.cigarstring = '40X'
+        self.assertEqual(a.infer_query_length(), 40)
+        a.cigarstring = '20M5I20M'
+        self.assertEqual(a.infer_query_length(), 45)
+        a.cigarstring = '20M5D20M'
+        self.assertEqual(a.infer_query_length(), 40)
+        a.cigarstring = '5H35M'
+        self.assertEqual(a.infer_query_length(), 35)
+        a.cigarstring = '5S35M'
+        self.assertEqual(a.infer_query_length(), 40)
+        a.cigarstring = '35M5H'
+        self.assertEqual(a.infer_query_length(), 35)
+        a.cigarstring = '35M5S'
+        self.assertEqual(a.infer_query_length(), 40)
+
+    def test_infer_read_length(self):
+        '''Test infer_read_length on M|=|X|I|D|H|S cigar ops'''
+        a = self.buildRead()
+        a.cigarstring = '40M'
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '40='
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '40X'
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '20M5I20M'
+        self.assertEqual(a.infer_read_length(), 45)
+        a.cigarstring = '20M5D20M'
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '5H35M'
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '5S35M'
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '35M5H'
+        self.assertEqual(a.infer_read_length(), 40)
+        a.cigarstring = '35M5S'
+        self.assertEqual(a.infer_read_length(), 40)
+
     def test_get_aligned_pairs_soft_clipping(self):
         a = self.buildRead()
         a.cigartuples = ((4, 2), (0, 35), (4, 3))
@@ -370,10 +414,28 @@ class TestAlignedSegment(ReadTest):
         self.assertEqual(a.query_alignment_length, 20)
         a.cigarstring = "20M1S"
         self.assertEqual(a.query_alignment_length, 20)
+        a.cigarstring = "20M1H"
+        self.assertEqual(a.query_alignment_length, 20)
         a.cigarstring = "1S20M"
         self.assertEqual(a.query_alignment_length, 20)
+        a.cigarstring = "1H20M"
+        self.assertEqual(a.query_alignment_length, 20)
         a.cigarstring = "1S20M1S"
         self.assertEqual(a.query_alignment_length, 20)
+        a.cigarstring = "1H20M1H"
+        self.assertEqual(a.query_alignment_length, 20)
+
+    def test_query_length_is_limited(self):
+        
+        a = self.buildRead()
+        a.query_name = "A" * 1
+        a.query_name = "A" * 251
+        self.assertRaises(
+            ValueError,
+            setattr,
+            a,
+            "query_name",
+            "A" * 252)
 
 
 class TestCigarStats(ReadTest):
@@ -754,5 +816,35 @@ class TestAsString(unittest.TestCase):
             for s, p in zip(reference, pysamf):
                 self.assertEqual(s, p.tostring(pysamf))
 
+
+class TestEnums(unittest.TestCase):
+    
+    def test_cigar_enums_are_defined(self):
+        self.assertEqual(pysam.CMATCH, 0)
+        self.assertEqual(pysam.CINS, 1)
+        self.assertEqual(pysam.CDEL, 2)
+        self.assertEqual(pysam.CREF_SKIP, 3)
+        self.assertEqual(pysam.CSOFT_CLIP, 4)
+        self.assertEqual(pysam.CHARD_CLIP, 5)
+        self.assertEqual(pysam.CPAD, 6)
+        self.assertEqual(pysam.CEQUAL, 7)
+        self.assertEqual(pysam.CDIFF, 8)
+        self.assertEqual(pysam.CBACK, 9)
+
+    def test_sam_flags_are_defined(self):
+        self.assertEqual(pysam.FPAIRED, 1)
+        self.assertEqual(pysam.FPROPER_PAIR, 2)
+        self.assertEqual(pysam.FUNMAP, 4)
+        self.assertEqual(pysam.FMUNMAP, 8)
+        self.assertEqual(pysam.FREVERSE, 16)
+        self.assertEqual(pysam.FMREVERSE, 32)
+        self.assertEqual(pysam.FREAD1, 64)
+        self.assertEqual(pysam.FREAD2, 128)
+        self.assertEqual(pysam.FSECONDARY, 256)
+        self.assertEqual(pysam.FQCFAIL, 512)
+        self.assertEqual(pysam.FDUP, 1024)
+        self.assertEqual(pysam.FSUPPLEMENTARY, 2048)
+
+
 if __name__ == "__main__":
     unittest.main()
