@@ -78,7 +78,6 @@ def check_samtools_view_equal(
     '''return true if the two files are equal in their
     content through samtools view.
     '''
-
     # strip MD and NM tags, as not preserved in CRAM files
     args = ["-x", "MD", "-x", "NM"]
     if not without_header:
@@ -161,8 +160,10 @@ def check_lines_equal(cls, a, b, sort=False, filter_f=None, msg=None):
     filter_f:
        remover lines in both a and b where expression is True
     """
-    aa = openfile(a).readlines()
-    bb = openfile(b).readlines()
+    with openfile(a) as inf:
+        aa = inf.readlines()
+    with openfile(b) as inf:
+        bb = inf.readlines()
 
     if filter_f is not None:
         aa = [x for x in aa if not filter_f(x)]
@@ -183,3 +184,28 @@ def get_temp_filename(suffix=""):
         dir=".")
     f.close()
     return f.name
+
+
+def load_and_convert(filename, encode=True):
+    '''load data from filename and convert all fields to string.
+
+    Filename can be either plain or compressed (ending in .gz).
+    '''
+    data = []
+    if filename.endswith(".gz"):
+        with gzip.open(filename) as inf:
+            for line in inf:
+                line = line.decode("ascii")
+                if line.startswith("#"):
+                    continue
+                d = line.strip().split("\t")
+                data.append(d)
+    else:
+        with open(filename) as f:
+            for line in f:
+                if line.startswith("#"):
+                    continue
+                d = line.strip().split("\t")
+                data.append(d)
+
+    return data
